.TH tina n "Version 2.7.0" "Tina Tools"

.SH NAME
tina - state space generation tool

Part of Tina Toolbox for analysis of Petri nets and Time Petri nets
(see also nd, struct, ktzio).

.SH SYNOPSIS

.B tina 
[\-\f2h\f1elp] 
     [\-C(|k|b) | \-R |
      \-V | \-Vq | \-V(o|f|l) tlist | \-P | \-Q(p|b|h) |
      \-M | \-Mm | \-E | \-Ea | \-W | \-Wm | \-S | \-Sa | \-A]
     [\-s n] [\-c n] [\-b n] [\-t s] [\-m n]
     [-df | -bf] [\-g | \-p]
     [\-v | \-q | \-\f2a\f1ut | \-mec | \-bcg | \-ktz] 
     [\-sp n] [\-tp n] [\-wp n]
     [infile] [outfile]

.SH DESCRIPTION

tina builds various  state space abstractions for Petri  nets and Time
Petri nets.   It takes as  input descriptions of  a Petri net  or Time
Petri net in  textual (.net format) or graphical  form (.ndr format of
files produced by nd).

.SH OPTIONS

.TP
\-\f2h\f1elp Recalls options.

.TP
Operating mode options:

.TP
\-C |  -Ck |  -Cb
Build the Coverability Graph of a Petri net (untimed, or with temporal
information discarded). Three heuristics are proposed:
-Cb is as fast as -R below if the  net is actually bounded, but may be
very slow if it is not;
-Ck  implements a  heuristic closer  to  the original  Karp and Miller
method, it may be very  slow  if  the net  is  actually  bounded,  but
unbounded places are detected as early as possible;
-C behaves like  -Cb until an unbounded marking  is found, after which
it behaves  like -Ck,  detecting further unbounded  places as  soon as
possible.

.TP
\-R
Builds the marking reachability graph of a Petri net (untimed, or with
temporal information discarded).

.TP
\-V | -Vq | -Vo tlist | -Vf tlist | -Vl tlist

Builds  the Covering  Step  Graph of  a  Petri net,  according to  the
technique of Vernadat/Azema/Michel.  -V preserves deadlocks. -Vq stops
as    soon   as    all   transitions    have   been    fired   (checks
quasi-liveness).  tlist being  a list  of observable  transitions, -Vo
tlist  preserves observational equivalence  wrt transitions  in tlist,
-Vf  tlist  preserves failure  equivalence,  and  -Vl tlist  preserves
linear  time temporal  properties. If  all transitions  are observable
then -V(o|f|l) tlist is eqv to -R.

.TP
\-P
Builds  the partial marking  graph of  a Petri  net, according  to the
persistent sets technique. The heuristic retained is to minimize local
branching.

.TP
\-Qp | -Qb | -Qh
Builds  the  partial  marking  graph  of  a  Petri  net.  Combine  the
persistent  sets  (-P)  and   covering  steps  (-V)  methods.  Several
heuristics are available:
-Qp maximizes size of steps;
-Qb steps minimizing branching, like -P;
-Qh largest conflict-free steps, a trade off between -Qp and -Qb.

.TP
\-W
Build the linear  state class graph of a Time  Petri net, according to
the technique  of Berthomieu/Menasche.   The linear state  class graph
preserves reachability properties and linear time temporal properties.
If no  temporal information is  specified, or if all  transitions bear
interval [0,w[, then -W is silently replaced by -R.

.TP
\-Wm
As \-W, except interprets multi-enabledness of transitions.

.TP
\-S
Build the strong linear state class  graph of a Time Petri net.  If no
temporal information is specified, or if all transitions bear interval
[0,w[, then -S is silently replaced by -R.

.TP
\-Sa
As  \-S, except  signals  non atomic  state  classes.

.TP
\-M | -Mm
As -W or -Wm, except only markings are preserved (firing sequences are not).

.TP
\-E | -Ea
As -S or -Sa, except only states are preserved (firing sequences are not).

.TP
\-A
Build the  arborescent, or atomic, state  class graph of  a Time Petri
net.  The arborescent state  class graph preserves liveness properties
and branching time temporal properties.  If no temporal information is
specified,  or if  all transitions  bear  interval [0,w[,  then -A  is
silently replaced by -R.

.TP
Exploration strategy flags:

.TP
\-df
Depth-first exploration (meaningless for -A)

.TP
\-bf
Breadth-first exploration (default for -A)


.TP
Stopping test and limits flags:

.TP
\-s n
Stopping test for the boundedness property.
.nf
\-s 0 : none, use limit option(s) to ensure termination
.fi
.nf
\-s 1 : Karp and Miller condition (default)
.fi
.nf
\-s 2 : \-s 1 + inclusion of firing domains
.fi
.nf
\-s 3 : \-s 2 + saturation of transitions following
       places with increasing markings (see documents).
.fi
.nf
With \-R, conditions \-s 2 and \-s 3 are equivalent to \-s 1
.fi
.nf
With \-C, no stopping test is necessary.
.fi

.TP
\-c n 
Bound on the number of markings to enumerate (none if n=0).

.TP
\-b n 
Bound on the marking of each place (none if n=0).

.TP
\-t n
Bound on duration of enumeration (none if n=0).

.TP
\-m n
Bound on multi-enabledness of transitions (none if n=0).

.TP
Output format selection :

.TP
\-v : textual output, prints full results

.TP
\-q : textual output, prints a summary of results

.TP
\-\f2a\f1ut  : transition  system output  in .aut  format, as  read by
Aldebaran.

.TP
\-mec : transition system output in .mec format, as read by Mec 4.

.TP
\-ktz : transition system output in \.ktz proprietary compact 
format.

.TP
\-bcg : transition system output in .bcg compact format,
as read by bcg tools.

.TP
\-sp n : encoding of state properties for transition system 
outputs:
.nf
-sp 0 : no state properties (default for .aut, .bcg)
.fi
.nf
-sp 1 : boolean state properties (default for .mec)
.fi
.nf
-sp 2 : weighted state properties (default for .ktz)
.fi
.nf
-sp 3 : weighted state properties for all weights
.fi

The .aut and .bcg formats do not natively support state properties. With -sp n,  where n>0, tina encodes them as additional transitions: regular transition properties (labels) are prefixed by  "T:", state  properties (markings) are  introduced as loops on states labelled by the places marked, prefixed by "S:". In  addition, when n>1, place names are suffixed by their marking, separated by "*".  E.g.  if place p3 has 3 tokens in state s, than -sp 1 adds transition (s,S:p3,s), -sp 2 adds  (s,S:p3*3,s), and -sp 3 adds  (s,S:p3,s), (s,S:p3*2,s) and (s,S:p3*3,s).

The .mec format natively supports state properties, but not weights. Weights are  encoded  as in the .aut  and  .bcg  outputs.  E.g. marking  p*3  is represented by state property  p with -sp 1, p*3 with  -sp 2, and properties p, p*2, and p*3, with -sp 3.

The .ktz format natively supports weighted state properties, even for infinite weights (unbounded places). For .ktz, flag values 1 to 3 yield the same result.

.TP
\-tp n : encoding of transition properties for transition system
outputs:
.nf
-tp 0 : no transition properties
.fi
.nf
-tp 1 : boolean transition properties (default for all)
.fi

.TP
\-wp n : encoding of wait states for transition system outputs:

.nf
-wp 0 : no wait state properties
.fi
.nf
-wp 1 : emit wait state properties
.fi

In Time Petri nets, it may happen that no transition firable from some state has a finite firing deadline. Knowledge of such "wait" states is important for e.g. LTL model checkers. With -wp 1, an extra "wait" property is emitted for all wait states (thus for all states if the net is untimed or all transitions have interval [0,w[). In all formats, it is represented by an extra transition labelled "*" looping on states having that property.

Default value: -wp 1 if at least one transition has an interval different from [0,w[, otherwise -wp 0. This treatment avoids to associate a wait property for all behavior states in untimed nets, for which LTL model checking typically assumes a fairness hypothesis for transition firing. The tina LTL model checkers has a flag to restore these properties, if desired.

.TP
Other flags:

.TP
\-g
Disables liveness analysis (if not -a or -C).

.TP
\-p
Disables reachability and liveness analysis (just parses).

.TP
Input source:

.TP
infile
Where the net is read, extension must  be .net or .ndr.  If absent or
specified by "-", then reads on standard input.

.TP
Output destination:

.TP
outfile

Where  results  are written,  into  a  format  deduced from  the  file
extension and  output flag (see correspondance  below). tina complains
in case of conflicts. If absent  or specified by "-", then results are
written  on standard  output in  the  format specified  by the  output
flags.

.nf
output flag     file extension          output format
.fi
.nf
--------------------------------------------------------------
.fi
.nf
-ktz            .ktz                    tina binary ktz format
.fi
.nf
-bcg            .bcg                    bcg tools bcg format
.fi
.nf
-aut            .aut                    aldebaran aut format
.fi
.nf
-mec            .mec                    mec4 mec format
.fi
.nf
-v | -q         any other               textual output
.fi

.TP
Default options:

tina -W -s 1 -c 0 -t 0 -b 0 -m 0 -v




.SH FILES

File nets/FORMATS  specifies syntax of  .net and .ndr  net description
files, and of .aut and .adr automata description files.


.SH EXAMPLES

.nf
tina -q  < nets/ifip.ndr
.fi
.nf
tina -s 2 nets/abp.net abp.txt
.fi
.nf
tina -R -q < nets/ifip.ndr > ifip.log-R
.fi
.nf
tina -a nets/ifip.ndr > ifip.aut
.fi
.nf
tina -C ../nets/hp.ndr hp.ktz
.fi


.SH SEE ALSO

nd, struct, ktzio

.SH AUTHORS
Bernard Berthomieu, LAAS/CNRS, 2000-2004, Bernard.Berthomieu@laas.fr.

